@@ -1,10 +1,11 @@
-use super::tokenizer::Token;
+use arguments;
+use tokenizer::Token;
 use std::env;
 use std::fs;
 use std::ffi::OsStr;
 
 pub enum Kind<'a> {
-    Tokens(&'a [Token<'a>]),
+    Tokens(&'a [Token]),
     Input(&'a str)
 }
 
@@ -43,3 +44,14 @@ pub fn dash_exists() -> bool {
     }
     false
 }
+
+/// Sets the corresponding flags if a shell is required and if dash exists.
+pub fn set_flags(flags: &mut u16, arguments: &[Token]) {
+    if required(Kind::Tokens(arguments)) {
+        if dash_exists() {
+            *flags |= arguments::SHELL_ENABLED + arguments::DASH_EXISTS;
+        } else {
+            *flags |= arguments::SHELL_ENABLED;
+        }
+    }
+}
